﻿using System;
using System.Collections.Generic;
using HeuristicLab.Algorithms.PushGP.Expressions;

using LinqExpression = System.Linq.Expressions.Expression;

namespace HeuristicLab.Algorithms.PushGP
{
    public static class ExpressionCreatorTable
    {
        private static Dictionary<OpCode, Func<Expression>> symbolTable = new Dictionary<OpCode, Func<Expression>>
        {
            #region Exec
            { OpCode.ExecPop, GetExpressionCreator(typeof(ExecPopExpression)) },
            { OpCode.ExecDup, GetExpressionCreator(typeof(ExecDuplicateExpression)) },
            { OpCode.ExecDefine, GetExpressionCreator(typeof(ExecDefineExpression)) },
            { OpCode.ExecDoXRange, GetExpressionCreator(typeof(ExecDoRangeExpression)) },
            { OpCode.ExecY, GetExpressionCreator(typeof(ExecYExpression)) },
            { OpCode.ExecIf, GetExpressionCreator(typeof(ExecIfExpression)) },
            #endregion

            #region Code
            { OpCode.CodeDup, GetExpressionCreator(typeof(CodeDuplicateExpression)) },
            { OpCode.CodeDefine, GetExpressionCreator(typeof(CodeDefineExpression)) },
            { OpCode.CodeQuote, GetExpressionCreator(typeof(CodeQuoteExpression)) },
            { OpCode.CodeDo, GetExpressionCreator(typeof(CodeDoExpression)) },
            { OpCode.CodeIf, GetExpressionCreator(typeof(CodeIfExpression)) },
            #endregion Code

            #region Name
            { OpCode.NamePop, GetExpressionCreator(typeof(NamePopExpression)) },
            { OpCode.NameDup, GetExpressionCreator(typeof(NameDuplicateExpression)) },
            #endregion Name

            #region Boolean
            { OpCode.BooleanPop, GetExpressionCreator(typeof(BooleanPopExpression)) },
            { OpCode.BooleanDup, GetExpressionCreator(typeof(BooleanDuplicateExpression)) },
            { OpCode.BooleanDefine, GetExpressionCreator(typeof(BooleanDefineExpression)) },
            { OpCode.BooleanOr, GetExpressionCreator(typeof(BooleanOrExpression)) },
            { OpCode.BooleanAnd, GetExpressionCreator(typeof(BooleanAndExpression)) },
            #endregion Boolean

            #region Integer
            { OpCode.IntegerPop, GetExpressionCreator(typeof(IntegerPopExpression)) },
            { OpCode.IntegerDup, GetExpressionCreator(typeof(IntegerDuplicateExpression)) },
            { OpCode.IntegerDefine, GetExpressionCreator(typeof(IntegerDefineExpression)) },
            { OpCode.IntegerMultiply, GetExpressionCreator(typeof(IntegerMultiplyExpression)) },
            { OpCode.IntegerDivide, GetExpressionCreator(typeof(IntegerDivideExpression)) },
            { OpCode.IntegerAdd, GetExpressionCreator(typeof(IntegerAddExpression)) },
            { OpCode.IntegerSubtract, GetExpressionCreator(typeof(IntegerSubtractExpression)) },
            { OpCode.IntegerMin, GetExpressionCreator(typeof(IntegerMinExpression)) },
            { OpCode.IntegerMax, GetExpressionCreator(typeof(IntegerMaxExpression)) },
            { OpCode.IntegerSmallerThan, GetExpressionCreator(typeof(IntegerSmallerThanExpression)) },
            { OpCode.IntegerGreaterThan, GetExpressionCreator(typeof(IntegerGreaterThanExpression)) },
            { OpCode.IntegerEquals, GetExpressionCreator(typeof(IntegerEqualsExpression)) },
            #endregion Integer

            #region Float
            { OpCode.FloatPop, GetExpressionCreator(typeof(FloatPopExpression)) },
            { OpCode.FloatDup, GetExpressionCreator(typeof(FloatDuplicateExpression)) },
            { OpCode.FloatDefine, GetExpressionCreator(typeof(FloatDefineExpression)) },
            { OpCode.FloatAdd, GetExpressionCreator(typeof(FloatAddExpression)) },
            { OpCode.FloatMultiply, GetExpressionCreator(typeof(FloatMultiplyExpression)) },
            { OpCode.FloatDivide, GetExpressionCreator(typeof(FloatDivideExpression)) },
            { OpCode.FloatSubtract, GetExpressionCreator(typeof(FloatSubtractExpression)) },
            { OpCode.FloatMin, GetExpressionCreator(typeof(FloatMinExpression)) },
            { OpCode.FloatMax, GetExpressionCreator(typeof(FloatMaxExpression)) },
            { OpCode.FloatSmallerThan, GetExpressionCreator(typeof(FloatSmallerThanExpression)) },
            { OpCode.FloatGreaterThan, GetExpressionCreator(typeof(FloatGreaterThanExpression)) },
            { OpCode.FloatEquals, GetExpressionCreator(typeof(FloatEqualsExpression)) },
            #endregion Float
        };

        private static Func<Expression> GetExpressionCreator(Type type)
        {
            return LinqExpression.Lambda<Func<Expression>>(
                       LinqExpression.New(type.GetConstructor(Type.EmptyTypes))
                   ).Compile();
        }

        public static Func<Expression> GetCreator(OpCode opCode)
        {
            Func<Expression> creator;
            if (symbolTable.TryGetValue(opCode, out creator)) return creator;
            else throw new NotSupportedException("OpCode: " + opCode);
        }

        public static bool TryGetCreator(OpCode opCode, out Func<Expression> creator)
        {
            return symbolTable.TryGetValue(opCode, out creator);
        }
    }
}
