﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Algorithms.PushGP
{
    public static class SymbolTable
    {
        private static Dictionary<string, OpCode> symbolTable = new Dictionary<string, OpCode>
        {
            #region Code
            { "CODE.NOOP", OpCode.CodeNoop },
            { "CODE.PUSH", OpCode.CodePush },
            { "CODE.IF", OpCode.CodeIf },
            { "CODE.DEFINE", OpCode.CodeDefine },
            { "CODE.DUP", OpCode.CodeDup },
            { "CODE.QUOTE", OpCode.CodeQuote },
            { "CODE.DO", OpCode.CodeDo },
            #endregion Code

            #region Exec
            { "EXEC.EXPAND", OpCode.ExecExpand },
            { "EXEC.PUSH", OpCode.ExecPush },
            { "EXEC.IF", OpCode.ExecIf },
            { "EXEC.DEFINE", OpCode.ExecDefine },
            { "EXEC.DUP", OpCode.ExecDup },
            { "EXEC.POP", OpCode.ExecPop },
            { "EXEC.Y", OpCode.ExecY },
            { "EXEC.DO*RANGE", OpCode.ExecDoXRange },
            #endregion Exec

            #region Name
            { "NAME.DEFINE*EXEC", OpCode.NameDefineXExec },
            { "NAME.POP", OpCode.NamePop },
            { "NAME.DUP", OpCode.NameDup },
            #endregion Name

            #region Boolean
            { "BOOLEAN.PUSH", OpCode.BooleanPush },
            { "BOOLEAN.POP", OpCode.BooleanPop },
            { "BOOLEAN.DUP", OpCode.BooleanDup },
            { "BOOLEAN.AND", OpCode.BooleanAnd },
            { "BOOLEAN.OR", OpCode.BooleanOr },
            #endregion Boolean

            #region Integer
            { "INTEGER.PUSH", OpCode.IntegerPush },
            { "INTEGER.POP", OpCode.IntegerPop },
            { "INTEGER.DUP", OpCode.IntegerDup },
            { "INTEGER.DEFINE", OpCode.IntegerDefine },
            { "INTEGER.+", OpCode.IntegerAdd },
            { "INTEGER.-", OpCode.IntegerSubtract },
            { "INTEGER.*", OpCode.IntegerMultiply },
            { "INTEGER./", OpCode.IntegerDivide },
            { "INTEGER.<", OpCode.IntegerSmallerThan },
            { "INTEGER.>", OpCode.IntegerGreaterThan },
            { "INTEGER.=", OpCode.IntegerEquals },
            { "INTEGER.MIN", OpCode.IntegerMin },
            { "INTEGER.MAX", OpCode.IntegerMax },
            #endregion Integer

            #region Float
            { "FLOAT.PUSH", OpCode.FloatPush },
            { "FLOAT.POP", OpCode.FloatPop },
            { "FLOAT.DUP", OpCode.FloatDup },
            { "FLOAT.DEFINE", OpCode.FloatDefine },
            { "FLOAT.+", OpCode.FloatAdd },
            { "FLOAT.-", OpCode.FloatSubtract },
            { "FLOAT.*", OpCode.FloatMultiply },
            { "FLOAT./", OpCode.FloatDivide },
            { "FLOAT.<", OpCode.FloatSmallerThan },
            { "FLOAT.>", OpCode.FloatGreaterThan },
            { "FLOAT.=", OpCode.FloatEquals },
            { "FLOAT.MIN", OpCode.FloatMin },
            { "FLOAT.MAX", OpCode.FloatMax },
            #endregion Float
        };

        public static OpCode GetOpCode(string operationName)
        {
            OpCode opCode;
            if (symbolTable.TryGetValue(operationName, out opCode)) return opCode;
            else throw new NotSupportedException("Symbol: " + operationName);
        }

        public static bool TryGetOpCode(string operationName, out OpCode opCode)
        {
            return symbolTable.TryGetValue(operationName, out opCode);
        }

        public static string GetSymbol(OpCode opCode)
        {
            var symbol = symbolTable.SingleOrDefault(pair => pair.Value == opCode);

            return !symbol.Equals(default(KeyValuePair<string, OpCode>))
                ? symbol.Key
                : null;
        }
    }
}
